@@ -114,7 +114,12 @@
 #'  angle). The base angle is computed in the same way but from the first base
 #'  pixel.
 #'
-#'@inheritParams image_binary
+#'    - If `width_at = TRUE`, the width at the  5th, 25th, 50th, 75th, and 95th
+#' percentiles of the object height are computed by default. These quantiles can
+#' be adjusted with the `width_at_percentiles` argument.
+#'
+#' @inheritParams image_binary
+#' @inheritParams image_index
 #'
 #'@param img The image to be analyzed.
 #'@param foreground,background A color palette for the foregrond and background,
@@ -125,6 +130,13 @@
 #'   pick up the color palettes for foreground and background for the image. If
 #'   `TRUE` [pick_palette()] will be called internally so that the user can sample
 #'   color points representing foreground and background.
+#' @param segment_objects Segment objects in the image? Defaults to `TRUE`. In
+#'   this case, objects are segmented using the index defined in the `index`
+#'   argument, and each object is analyzed individually. If `segment_objects =
+#'   FALSE` is used, the objects are not segmented and the entire image is
+#'   analyzed. This is useful, for example, when analyzing an image without
+#'   background, where an `object_index` could be computed for the entire image,
+#'   like the index of a crop canopy.
 #' @param viewer The viewer option. This option controls the type of viewer to
 #'   use for interactive plotting (eg., when `pick_palettes = TRUE`).  If not
 #'   provided, the value is retrieved using [get_pliman_viewer()].
@@ -167,7 +179,7 @@
 #'  processing time when an image has lots of objects (say >1000).
 #'@param workers A positive numeric scalar or a function specifying the number
 #'  of parallel processes that can be active at the same time. By default, the
-#'  number of sections is set up to 50% of available cores.
+#'  number of sections is set up to 30% of available cores.
 #'@param watershed If `TRUE` (default) performs watershed-based object
 #'  detection. This will detect objects even when they are touching one other.
 #'  If `FALSE`, all pixels for each connected set of foreground pixels are set
@@ -189,6 +201,13 @@
 #'   object for which the angle should be computed (from the apex and the
 #'   bottom). Defaults to c(0.25, 0.75), which means considering the 25th and
 #'   75th percentiles of the object height.
+#' @param width_at Logical. If `TRUE`, the widths of the object at a given set
+#'   of quantiles of the height are computed.
+#' @param width_at_percentiles  A vector of heights along the vertical axis of
+#'   the object at which the width will be computed. The default value is
+#'   c(0.05, 0.25, 0.5, 0.75, 0.95), which means the function will return the
+#'   width at the 5th, 25th, 50th, 75th, and 95th percentiles of the object's
+#'   height.
 #'@param haralick Logical value indicating whether Haralick features are
 #'  computed. Defaults to `FALSE`.
 #'@param har_nbins An integer indicating the number of bins using to compute the
@@ -519,6 +538,7 @@ analyze_objects <- function(img,
                             foreground = NULL,
                             background = NULL,
                             pick_palettes = FALSE,
+                            segment_objects = TRUE,
                             viewer = get_pliman_viewer(),
                             reference = FALSE,
                             reference_area = NULL,
@@ -534,6 +554,8 @@ analyze_objects <- function(img,
                             sigma_veins = 1,
                             ab_angles = FALSE,
                             ab_angles_percentiles = c(0.25, 0.75),
+                            width_at = FALSE,
+                            width_at_percentiles = c(0.05, 0.25, 0.50, 0.75, 0.95),
                             haralick = FALSE,
                             har_nbins = 32,
                             har_scales = 1,
@@ -547,6 +569,11 @@ analyze_objects <- function(img,
                             invert = FALSE,
                             object_size = "medium",
                             index = "NB",
+                            r = 1,
+                            g = 2,
+                            b = 3,
+                            re = 4,
+                            nir = 5,
                             object_index = NULL,
                             pixel_level_index = FALSE,
                             return_mask = FALSE,
@@ -615,7 +642,7 @@ analyze_objects <- function(img,
              tolerance, extension, randomize, nrows, plot, show_original,
              show_background, marker, marker_col, marker_size, save_image,
              prefix, dir_original, dir_processed, verbose, col_background,
-             col_foreground, lower_noise, ab_angles, ab_angles_percentiles, return_mask, pcv){
+             col_foreground, lower_noise, ab_angles, ab_angles_percentiles, width_at, width_at_percentiles, return_mask, pcv){
       if(is.character(img)){
         all_files <- sapply(list.files(diretorio_original), file_name)
         check_names_dir(img, all_files, diretorio_original)
@@ -661,7 +688,7 @@ analyze_objects <- function(img,
                                        title = "Use the first mouse button to pick up BACKGROUND colors. Click 'Done' to finish",
                                        viewer = viewer)
             if(viewopt != "base"){
-              image_view(img |> reduce_dimensions(2000))
+              image_view(img[1:10, 1:10,])
             }
             if(viewopt == "base"){
               message("Use the first mouse button to pick up FOREGROUND colors. Press Est to exit")
@@ -742,30 +769,42 @@ analyze_objects <- function(img,
           }
 
         } else{
-          img2 <- help_binary(img,
-                              index = index,
-                              invert = invert,
-                              fill_hull = fill_hull,
-                              threshold = threshold,
-                              k = k,
-                              windowsize = windowsize,
-                              filter = filter,
-                              resize = FALSE)
-          if(isTRUE(watershed)){
-            parms <- read.csv(file=system.file("parameters.csv", package = "pliman", mustWork = TRUE), header = T, sep = ";")
-            res <- length(img2)
-            parms2 <- parms[parms$object_size == object_size,]
-            rowid <-
-              which(sapply(as.character(parms2$resolution), function(x) {
-                eval(parse(text=x))}))
-            ext <- ifelse(is.null(extension),  parms2[rowid, 3], extension)
-            tol <- ifelse(is.null(tolerance), parms2[rowid, 4], tolerance)
-            nmask <- EBImage::watershed(EBImage::distmap(img2),
-                                        tolerance = tol,
-                                        ext = ext)
+          if(isTRUE(segment_objects)){
+            img2 <- help_binary(img,
+                                index = index,
+                                r = r,
+                                g = g,
+                                b = b,
+                                re = re,
+                                nir = nir,
+                                invert = invert,
+                                fill_hull = fill_hull,
+                                threshold = threshold,
+                                k = k,
+                                windowsize = windowsize,
+                                filter = filter,
+                                resize = FALSE)
+            if(isTRUE(watershed)){
+              parms <- read.csv(file=system.file("parameters.csv", package = "pliman", mustWork = TRUE), header = T, sep = ";")
+              res <- length(img2)
+              parms2 <- parms[parms$object_size == object_size,]
+              rowid <-
+                which(sapply(as.character(parms2$resolution), function(x) {
+                  eval(parse(text=x))}))
+              ext <- ifelse(is.null(extension),  parms2[rowid, 3], extension)
+              tol <- ifelse(is.null(tolerance), parms2[rowid, 4], tolerance)
+              nmask <- EBImage::watershed(EBImage::distmap(img2),
+                                          tolerance = tol,
+                                          ext = ext)
+            } else{
+              nmask <- EBImage::bwlabel(img2)
+            }
           } else{
+            img2 <- img[,,1]
+            img2[img2@.Data == 0 | img2@.Data != 0] <- TRUE
             nmask <- EBImage::bwlabel(img2)
           }
+
           ID <- which(img2 == 1)
           ID2 <- which(img2 == 0)
         }
@@ -799,6 +838,11 @@ analyze_objects <- function(img,
                         threshold = threshold,
                         index = back_fore_index,
                         filter = filter,
+                        r = r,
+                        g = g,
+                        b = b,
+                        re = re,
+                        nir = nir,
                         k = k,
                         windowsize = windowsize,
                         invert = invert1,
@@ -819,6 +863,11 @@ analyze_objects <- function(img,
             help_binary(img3,
                         threshold = threshold,
                         index = fore_ref_index,
+                        r = r,
+                        g = g,
+                        b = b,
+                        re = re,
+                        nir = nir,
                         filter = filter,
                         k = k,
                         windowsize = windowsize,
@@ -963,6 +1012,11 @@ analyze_objects <- function(img,
               help_binary(img,
                           threshold = threshold,
                           index = index,
+                          r = r,
+                          g = g,
+                          b = b,
+                          re = re,
+                          nir = nir,
                           k = k,
                           windowsize = windowsize,
                           filter = filter,
@@ -1080,8 +1134,21 @@ analyze_objects <- function(img,
       # check if angles should be computed
       if(isTRUE(ab_angles)){
         angles <- poly_apex_base_angle(object_contour, ab_angles_percentiles)
+    } else{
+      angles <- NULL
+    }
+
+      # check if width at should be computed
+      if(isTRUE(width_at)){
+        widths <-
+          do.call(rbind, lapply(object_contour, function(x){
+            x |> poly_align(plot = FALSE) |> poly_width_at(width_at_percentiles)
+          })) |>
+          as.data.frame() |>
+          rownames_to_column("id")
+        names(widths) <- c("id", paste0("width", width_at_percentiles))
       } else{
-        angles <- NULL
+        widths <- NULL
       }
 
       # check if veins is computed
@@ -1105,7 +1172,7 @@ analyze_objects <- function(img,
         }
         ind <- read.csv(file=system.file("indexes.csv", package = "pliman", mustWork = TRUE), header = T, sep = ";")
         if(any(object_index %in% ind$Index)){
-          ind_formula <- ind[which(ind$Index %in% object_index), 2]
+          ind_formula <- ind[match(object_index, ind$Index), 2]
         } else{
           ind_formula <- object_index
         }
@@ -1122,7 +1189,7 @@ analyze_objects <- function(img,
                data.frame(
                  do.call(cbind,
                          lapply(seq_along(ind_formula), function(i){
-                           data.frame(transform(x, index = eval(parse(text = ind_formula[i])))[,8])
+                           data.frame(transform(x, index = eval(parse(text = ind_formula[i])))[,ncol(obj_rgb)+1])
                          })
                  )
                )
@@ -1169,8 +1236,10 @@ analyze_objects <- function(img,
                       efourier_minharm = min_harm,
                       veins = prop_veins,
                       angles = angles,
+                      width_at = widths,
                       mask = mask,
                       pcv = pcv,
+                      contours = object_contour,
                       parms = list(index = index))
       class(results) <- "anal_obj"
       if(plot == TRUE | save_image == TRUE){
@@ -1193,7 +1262,8 @@ analyze_objects <- function(img,
         }
 
         if(show_original == TRUE & show_segmentation == FALSE){
-          im2 <- img
+          im2 <- img[,,1:3]
+          EBImage::colorMode(im2) <- "Color"
           if(backg){
             im3 <- EBImage::colorLabels(nmask)
             im2@.Data[,,1][which(im3@.Data[,,1]==0)] <- col_background[1]
@@ -1220,7 +1290,8 @@ analyze_objects <- function(img,
             im2@.Data[,,2][which(im2@.Data[,,2]==0)] <- col_background[2]
             im2@.Data[,,3][which(im2@.Data[,,3]==0)] <- col_background[3]
           } else{
-            im2 <- img
+            im2 <- img[,,1:3]
+            EBImage::colorMode(im2) <- "Color"
             im2@.Data[,,1][ID] <- col_foreground[1]
             im2@.Data[,,2][ID] <- col_foreground[2]
             im2@.Data[,,3][ID] <- col_foreground[3]
@@ -1323,14 +1394,14 @@ analyze_objects <- function(img,
         }
       }
       invisible(results)
-    }
+}
 
   if(missing(pattern)){
     help_count(img, foreground, background, pick_palettes, resize, fill_hull, threshold, filter,
                tolerance , extension, randomize, nrows, plot, show_original,
                show_background, marker, marker_col, marker_size, save_image, prefix,
                dir_original, dir_processed, verbose, col_background,
-               col_foreground, lower_noise, ab_angles, ab_angles_percentiles, return_mask, pcv)
+               col_foreground, lower_noise, ab_angles, ab_angles_percentiles, width_at, width_at_percentiles, return_mask, pcv)
   } else{
     if(pattern %in% c("0", "1", "2", "3", "4", "5", "6", "7", "8", "9")){
       pattern <- "^[0-9].*$"
@@ -1348,7 +1419,7 @@ analyze_objects <- function(img,
     }
     if(parallel == TRUE){
       init_time <- Sys.time()
-      nworkers <- ifelse(is.null(workers), trunc(detectCores()*.5), workers)
+      nworkers <- ifelse(is.null(workers), trunc(detectCores()*.3), workers)
       cl <- parallel::makePSOCKcluster(nworkers)
       doParallel::registerDoParallel(cl)
       on.exit(stopCluster(cl))
@@ -1366,7 +1437,7 @@ analyze_objects <- function(img,
                      tolerance , extension, randomize, nrows, plot, show_original,
                      show_background, marker, marker_col, marker_size, save_image, prefix,
                      dir_original, dir_processed, verbose, col_background,
-                     col_foreground, lower_noise, ab_angles, ab_angles_percentiles, return_mask, pcv)
+                     col_foreground, lower_noise, ab_angles, ab_angles_percentiles, width_at, width_at_percentiles, return_mask, pcv)
         }
 
     } else{
@@ -1381,7 +1452,7 @@ analyze_objects <- function(img,
                    tolerance , extension, randomize, nrows, plot, show_original,
                    show_background, marker, marker_col, marker_size, save_image, prefix,
                    dir_original, dir_processed, verbose, col_background,
-                   col_foreground, lower_noise, ab_angles, ab_angles_percentiles, return_mask, pcv)
+                   col_foreground, lower_noise, ab_angles, ab_angles_percentiles, width_at, width_at_percentiles, return_mask, pcv)
       }
       results <-
         lapply(seq_along(names_plant), function(i){
@@ -1522,6 +1593,20 @@ analyze_objects <- function(img,
       angles <- NULL
     }
 
+    if(isTRUE(width_at)){
+      width_at <-
+        do.call(rbind,
+                lapply(seq_along(results), function(i){
+                  transform(results[[i]][["width_at"]],
+                            img =  names(results[i]))
+                })
+        )
+
+      width_at <- width_at[, c(ncol(width_at), 1:ncol(width_at)-1)]
+    } else{
+      width_at <- NULL
+    }
+
     if(isTRUE(pcv)){
       pcv <-
         do.call(rbind,
@@ -1571,12 +1656,13 @@ analyze_objects <- function(img,
              efourier_minharm = efourier_minharm,
              veins = veins,
              angles = angles,
+             width_at = width_at,
              pcv = pcv),
         class = "anal_obj_ls"
       )
     )
   }
-}
+  }
 
 
 #' @name analyze_objects
