@@ -394,6 +394,10 @@ get_measures <- function(object,
       if(!is.null(object$angles)){
         res <- cbind(res, object$angles[, -c(1:2)])
       }
+      # bind width_at if it exists
+      if(!is.null(object$width_at)){
+        res <- cbind(res, object$width_at[, -c(1:2)])
+      }
       # bind perimeter complexity value if it exists
       if(!is.null(object$pcv)){
         res <- cbind(res, pcv = object$pcv)
@@ -434,7 +438,7 @@ get_measures <- function(object,
              index = aggr)
     }
     class(out) <- c("measures_ls")
-    return(out)
+    invisible(out)
   } else{
 
     # bind object_index, if it exists
@@ -457,6 +461,11 @@ get_measures <- function(object,
       res <- cbind(res, object$angles[, -1])
     }
 
+    # bind width_at if it exists
+    if(!is.null(object$width_at)){
+      res <- cbind(res, object$width_at[, -1])
+    }
+
     # bind perimeter complexity value if it exists
     if(!is.null(object$pcv)){
       res <- cbind(res, pcv = object[["pcv"]])
@@ -464,7 +473,7 @@ get_measures <- function(object,
 
     res <- round_cols(res, digits = digits)
     class(res) <- c("data.frame", "measures")
-    return(res)
+    invisible(res)
   }
 }
 
@@ -858,7 +867,7 @@ features_moment <- function(x){
   }))
   res <- cbind(mc, moms)
   colnames(res) <- c("mx", "my", "maj_axis", "min_axis", "eccentricity", "theta")
-  return(data.frame(res))
+  invisible(data.frame(res))
 }
 
 features_shape <- function(x){
@@ -877,7 +886,7 @@ features_shape <- function(x){
                     radius_min = rmin,
                     radius_max = rmax,
                     radius_sd = rsd)
-  return(res)
+  invisible(res)
 }
 
 ## helper function to compute the measures based on a mask
@@ -968,7 +977,7 @@ compute_measures <- function(mask,
     shape <- cbind(shape, hal[valid, ])
     colnames(shape) <- c(names_measures(), har_names())
   }
-  return(list(shape = shape,
+  invisible(list(shape = shape,
               cont = ocont,
               ch = ch))
 }
@@ -1009,7 +1018,7 @@ compute_measures_disease <- function(mask){
                      "min_axis",
                      "length",
                      "width")]
-  return(list(shape = shape,
+  invisible(list(shape = shape,
               cont = ocont))
 }
 
